=== modified file 'plugin/drizzle_protocol/drizzle_protocol.cc'
--- plugin/drizzle_protocol/drizzle_protocol.cc	2010-12-07 18:30:53 +0000
+++ plugin/drizzle_protocol/drizzle_protocol.cc	2010-12-09 14:52:56 +0000
@@ -53,6 +53,8 @@
 
 static const uint32_t DRIZZLE_TCP_PORT= 4427;
 
+ProtocolCounters *ListenDrizzleProtocol::drizzle_counters= new ProtocolCounters();
+
 ListenDrizzleProtocol::~ListenDrizzleProtocol()
 {
 }
@@ -77,6 +79,8 @@
   context.registerVariable(new sys_var_const_string_val("bind_address",
                                                         vm["bind-address"].as<std::string>()));
 
+  context.registerVariable(new sys_var_uint32_t_ptr("max-connections", &ListenDrizzleProtocol::drizzle_counters->max_connections));
+
   return 0;
 }
 
@@ -104,6 +108,9 @@
   context("bind-address",
           po::value<std::string>()->default_value(""),
           N_("Address to bind to."));
+  context("max-connections",
+          po::value<uint32_t>(&ListenDrizzleProtocol::drizzle_counters->max_connections)->default_value(1000),
+          N_("Maximum simultaneous connections."));
 }
 
 } /* namespace drizzle_protocol */

=== modified file 'plugin/drizzle_protocol/drizzle_protocol.h'
--- plugin/drizzle_protocol/drizzle_protocol.h	2010-12-03 19:56:07 +0000
+++ plugin/drizzle_protocol/drizzle_protocol.h	2010-12-06 19:36:53 +0000
@@ -44,8 +44,9 @@
   { }
 
   ~ListenDrizzleProtocol();
-
   in_port_t getPort(void) const;
+  static ProtocolCounters *drizzle_counters;
+  virtual ProtocolCounters *getCounters(void) const { return drizzle_counters; }
 };
 
 

=== modified file 'plugin/mysql_protocol/mysql_protocol.cc'
--- plugin/mysql_protocol/mysql_protocol.cc	2010-12-03 23:18:06 +0000
+++ plugin/mysql_protocol/mysql_protocol.cc	2010-12-06 19:36:53 +0000
@@ -57,6 +57,8 @@
 static const double random_max_double= (double)0x3FFFFFFF;
 
 
+ProtocolCounters *ListenMySQLProtocol::mysql_counters= new ProtocolCounters();
+
 ListenMySQLProtocol::~ListenMySQLProtocol()
 { }
 
@@ -77,15 +79,12 @@
   if (new_fd == -1)
     return NULL;
 
-  return new ClientMySQLProtocol(new_fd, _using_mysql41_protocol);
+  return new ClientMySQLProtocol(new_fd, _using_mysql41_protocol, getCounters());
 }
 
-drizzled::atomic<uint64_t> ClientMySQLProtocol::connectionCount;
-drizzled::atomic<uint64_t> ClientMySQLProtocol::failedConnections;
-drizzled::atomic<uint64_t> ClientMySQLProtocol::connected;
-
-ClientMySQLProtocol::ClientMySQLProtocol(int fd, bool using_mysql41_protocol):
-  _using_mysql41_protocol(using_mysql41_protocol)
+ClientMySQLProtocol::ClientMySQLProtocol(int fd, bool using_mysql41_protocol, ProtocolCounters *set_counters):
+  _using_mysql41_protocol(using_mysql41_protocol),
+  counters(set_counters)
 {
   
   net.vio= 0;
@@ -143,7 +142,7 @@
   { 
     drizzleclient_net_close(&net);
     drizzleclient_net_end(&net);
-    connected.decrement();
+    counters->connected.decrement();
   }
 }
 
@@ -151,8 +150,8 @@
 {
   bool connection_is_valid;
 
-  connectionCount.increment();
-  connected.increment();
+  counters->connectionCount.increment();
+  counters->connected.increment();
 
   /* Use "connect_timeout" value during connection phase */
   drizzleclient_net_set_read_timeout(&net, connect_timeout.get());
@@ -161,13 +160,20 @@
   connection_is_valid= checkConnection();
 
   if (connection_is_valid)
-    sendOK();
+    if (counters->connected > counters->max_connections)
+    {
+      std::string errmsg(ER(ER_CON_COUNT_ERROR));
+      sendError(ER_CON_COUNT_ERROR, errmsg.c_str());
+    }
+    else
+      sendOK();
   else
   {
     sendError(session->main_da.sql_errno(), session->main_da.message());
-    failedConnections.increment();
+    counters->failedConnections.increment();
     return false;
   }
+
   /* Connect completed, set read/write timeouts back to default */
   drizzleclient_net_set_read_timeout(&net, read_timeout.get());
   drizzleclient_net_set_write_timeout(&net, write_timeout.get());
@@ -923,6 +929,8 @@
   context.registerVariable(new sys_var_const_string_val("bind_address",
                                                         vm["bind-address"].as<std::string>()));
 
+  context.registerVariable(new sys_var_uint32_t_ptr("max-connections", &ListenMySQLProtocol::mysql_counters->max_connections));
+
   return 0;
 }
 
@@ -950,13 +958,16 @@
   context("bind-address",
           po::value<string>()->default_value(""),
           N_("Address to bind to."));
+  context("max-connections",
+          po::value<uint32_t>(&ListenMySQLProtocol::mysql_counters->max_connections)->default_value(1000),
+          N_("Maximum simultaneous connections."));
 }
 
 static int mysql_protocol_connection_count_func(drizzle_show_var *var, char *buff)
 {
   var->type= SHOW_LONGLONG;
   var->value= buff;
-  *((uint64_t *)buff)= ClientMySQLProtocol::connectionCount;
+  *((uint64_t *)buff)= ListenMySQLProtocol::mysql_counters->connectionCount;
   return 0;
 }
 
@@ -964,7 +975,7 @@
 {
   var->type= SHOW_LONGLONG;
   var->value= buff;
-  *((uint64_t *)buff)= ClientMySQLProtocol::connected;
+  *((uint64_t *)buff)= ListenMySQLProtocol::mysql_counters->connected;
   return 0;
 }
 
@@ -972,7 +983,7 @@
 {
   var->type= SHOW_LONGLONG;
   var->value= buff;
-  *((uint64_t *)buff)= ClientMySQLProtocol::failedConnections;
+  *((uint64_t *)buff)= ListenMySQLProtocol::mysql_counters->failedConnections;
   return 0;
 }
 

=== modified file 'plugin/mysql_protocol/mysql_protocol.h'
--- plugin/mysql_protocol/mysql_protocol.h	2010-12-05 20:05:39 +0000
+++ plugin/mysql_protocol/mysql_protocol.h	2010-12-09 14:52:56 +0000
@@ -29,6 +29,17 @@
 
 namespace drizzle_plugin
 {
+class ProtocolCounters
+{
+  public:
+    ProtocolCounters():
+      max_connections(1000)
+    { }
+    drizzled::atomic<uint64_t> connectionCount;
+    drizzled::atomic<uint64_t> failedConnections;
+    drizzled::atomic<uint64_t> connected;
+    uint32_t max_connections;
+};
 
 typedef drizzled::constrained_check<uint32_t, 300, 1> timeout_constraint;
 typedef drizzled::constrained_check<uint32_t, 300, 1> retry_constraint;
@@ -52,6 +63,8 @@
   virtual const std::string getHost(void) const;
   virtual in_port_t getPort(void) const;
   virtual drizzled::plugin::Client *getClient(int fd);
+  static ProtocolCounters *mysql_counters;
+  virtual ProtocolCounters *getCounters(void) const { return mysql_counters; }
 };
 
 class ClientMySQLProtocol: public drizzled::plugin::Client
@@ -69,12 +82,10 @@
   void makeScramble(char *scramble);
 
 public:
-  ClientMySQLProtocol(int fd, bool _using_mysql41_protocol);
+  ClientMySQLProtocol(int fd, bool _using_mysql41_protocol, ProtocolCounters *set_counters);
   virtual ~ClientMySQLProtocol();
 
-  static drizzled::atomic<uint64_t> connectionCount;
-  static drizzled::atomic<uint64_t> failedConnections;
-  static drizzled::atomic<uint64_t> connected;
+  ProtocolCounters *counters;
 
   virtual int getFileDescriptor(void);
   virtual bool isConnected();

=== modified file 'plugin/mysql_unix_socket_protocol/protocol.cc'
--- plugin/mysql_unix_socket_protocol/protocol.cc	2010-12-03 19:56:07 +0000
+++ plugin/mysql_unix_socket_protocol/protocol.cc	2010-12-06 19:36:53 +0000
@@ -51,6 +51,8 @@
 
 static bool clobber= false;
 
+ProtocolCounters *Protocol::mysql_unix_counters= new ProtocolCounters();
+
 Protocol::~Protocol()
 {
   fs::remove(_unix_socket_path);
@@ -68,11 +70,13 @@
   fs::path uds_path(vm["path"].as<fs::path>());
   if (not fs::exists(uds_path))
   {
-    context.add(new Protocol("mysql_unix_socket_protocol",
+    Protocol *listen_obj= new Protocol("mysql_unix_socket_protocol",
                              true,
-                             uds_path));
+                             uds_path);
+    context.add(listen_obj);
     context.registerVariable(new sys_var_const_string_val("path", fs::system_complete(uds_path).file_string()));
     context.registerVariable(new sys_var_bool_ptr_readonly("clobber", &clobber));
+    context.registerVariable(new sys_var_uint32_t_ptr("max-connections", &Protocol::mysql_unix_counters->max_connections));
   }
   else
   {
@@ -155,7 +159,9 @@
           N_("Path used for MySQL UNIX Socket Protocol."));
   context("clobber",
           N_("Clobber socket file if one is there already."));
-
+  context("max-connections",
+          po::value<uint32_t>(&Protocol::mysql_unix_counters->max_connections)->default_value(1000),
+          N_("Maximum simultaneous connections."));
 }
 
 } /* namespace mysql_unix_socket_protocol */

=== modified file 'plugin/mysql_unix_socket_protocol/protocol.h'
--- plugin/mysql_unix_socket_protocol/protocol.h	2010-12-03 19:56:07 +0000
+++ plugin/mysql_unix_socket_protocol/protocol.h	2010-12-06 19:36:53 +0000
@@ -52,6 +52,8 @@
   bool getFileDescriptors(std::vector<int> &fds);
 
   in_port_t getPort(void) const;
+  static ProtocolCounters *mysql_unix_counters;
+  virtual ProtocolCounters *getCounters(void) const {return mysql_unix_counters; }
 };
 
 

